@@ -41,6 +41,11 @@
 #include <netdb.h>
 #endif
 
+#ifdef HAVE_LIB_DNET
+#include <netdnet/dn.h>
+#include <netdnet/dnetdb.h>
+#endif
+
 #define MUROAR_IOBUF         128
 
 #define _MU_STR              "\265"
@@ -105,6 +110,12 @@ static int muroar_open_socket (char * server) {
  struct sockaddr_un   un;
 #endif
  int fh = -1;
+#ifdef HAVE_LIB_DNET
+ char * buf, * node, * object;
+ static char localnode[16] = {0};
+ struct dn_naddr      *binaddr;
+ struct nodeent       *dp;
+#endif
 
 #ifdef __WIN32
  muroar_init_win32();
@@ -130,6 +141,52 @@ static int muroar_open_socket (char * server) {
 #else
   return -1;
 #endif
+ } else if ( strstr(server, "::") != NULL ) {
+#ifdef HAVE_LIB_DNET
+  // alloc a temp buffer so we can change the string at will:
+  buf = strdup(server);
+
+  // cut node::object into buf and object
+  object  = strstr(buf, "::");
+  *object = 0;
+   object += 2;
+
+  // use default if we have a zero-size node name:
+  if ( *buf == 0 ) {
+   if ( !localnode[0] ) {
+    if ( (binaddr=getnodeadd()) == NULL) {
+     free(buf);
+     return -1;
+    }
+
+    if ( (dp = getnodebyaddr((char*)binaddr->a_addr, binaddr->a_len, AF_DECnet)) == NULL ) {
+     free(buf);
+     return -1;
+    }
+
+    strncpy(localnode, dp->n_name, sizeof(localnode)-1);
+    localnode[sizeof(localnode)-1] = 0;
+   }
+
+   node = localnode;
+  } else {
+   node = buf;
+  }
+
+  // use default if we have a zero size object name:
+  if ( *object == 0 ) {
+   object = MUROAR_OBJECT;
+  }
+
+  fh = dnet_conn(node, object, SOCK_STREAM, NULL, 0, NULL, 0);
+
+  // free buffer when we are done.
+  free(buf);
+
+  return fh;
+#else
+  return -1;
+#endif
  }
 
  if ( (he = gethostbyname(server)) == NULL ) {
@@ -186,9 +243,11 @@ int muroar_connect(char * server, char * name) {
   if ( (fh = muroar_open_socket(server)) == -1 )
    return -1;
  } else {
+  // build string for ~/.roar
   snprintf(useraddr, sizeof(useraddr), "/%s/.roar", getenv("HOME"));
   useraddr[sizeof(useraddr)-1] = 0;
 
+  // go thru list of possible defaults:
   for (i = 0; fh == -1 && addr[i] != NULL; i++) {
    fh = muroar_open_socket(addr[i]);
   }
@@ -311,28 +370,6 @@ int muroar_quit   (int fh) {
  return ret;
 }
 
-// Send notify beep.
-// We only support no-data default beeps.
-int muroar_beep   (int fh) {
- unsigned char beep[] = "\0\30\0\0\0\0\0\0\0\0"; // BEEP command
-
- if ( fh == -1 ) {
-  _SET_ERRNO(EBADF);
-  return -1;
- }
-
- if ( muroar_write(fh, beep, 10) != 10 )
-  return -1;
-
- if ( muroar_read(fh, beep, 10) != 10 )
-  return -1;
-
- if ( beep[1] != MUROAR_CMD_OK )
-  return -1;
-
- return 0;
-}
-
 // Open a Stream
 int muroar_stream(int fh, int dir, int * stream, int codec, int rate, int channels, int bits) {
  unsigned char buf[24];
@@ -388,6 +425,8 @@ int muroar_stream(int fh, int dir, int * stream, int codec, int rate, int channe
  // Send EXEC_STREAM command:
  memset(buf, 0, 10);
  buf[1] = MUROAR_CMD_EXEC_STREAM;
+
+ // set Stream ID:
  buf[2] = (sid & 0xFF00) >> 8;
  buf[3] = (sid & 0x00FF);
 
